using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainForm()

        /// <summary>
        /// 
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.dataSourceComboBox.SelectedIndexChanged  += dataSourceComboBox_SelectedIndexChanged;
            this.colorSchemeComboBox.SelectedIndexChanged += colorSchemeComboBox_SelectedIndexChanged;
            this.rasterTypeComboBox.SelectedIndexChanged  += rasterTypeComboBox_SelectedIndexChanged;
            this.resetPositionButton.Click                += resetPositionButton_Click;
            this.saveImageButton.Click                    += saveImageButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region ε óϱ - OnLoad(e)

        /// <summary>
        /// ε óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.dataSourceComboBox.SelectedIndex = 0;

            this.colorSchemeComboBox.Sorted = false;

            foreach(ColoeScheme schemaType in Enum.GetValues(typeof(ColoeScheme)))
            {
                this.colorSchemeComboBox.Items.Add(schemaType);
            }

            this.colorSchemeComboBox.SelectedIndex = (int)ColoeScheme.Rainbow1;

            this.rasterTypeComboBox.Sorted = false;

            foreach(RasterType rasterType in Enum.GetValues(typeof(RasterType)))
            {
                this.rasterTypeComboBox.Items.Add(rasterType);
            }

            this.rasterTypeComboBox.SelectedIndex = (int)RasterType.Label;

            this.chart3DControl.AssignTrackBars(this.rhoTrackBar, this.thetaTrackBar, this.phiTrackBar);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region  ҽ ޺ ڽ  ε  óϱ - dataSourceComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        ///  ҽ ޺ ڽ  ε  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void dataSourceComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.chart3DControl.AxisXLegend = null;
            this.chart3DControl.AxisYLegend = null;
            this.chart3DControl.AxisZLegend = null;

            switch(this.dataSourceComboBox.SelectedIndex)
            {
                case 0: SetFunction1();          break;
                case 1: SetFunction2();          break;
                case 2: SetSurfacePoints();      break;
                case 3: SetScatterPoints(false); break;
                case 4: SetScatterPoints(true);  break;
                case 5: SetScatterPoints();      break;
            }

            this.informationLabel.Text = "Ʈ  : " + this.chart3DControl.PointCount;
        }

        #endregion
        #region  ǥ ޺ ڽ  ε  óϱ - colorSchemeComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        ///  ǥ ޺ ڽ  ε  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void colorSchemeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Color[] colorArray = ColorSchema.GetSchema((ColoeScheme)this.colorSchemeComboBox.SelectedIndex);

            this.chart3DControl.SetColorScheme(colorArray, 3);
        }

        #endregion
        #region  Ÿ ޺ ڽ  ε  óϱ - rasterTypeComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        ///  Ÿ ޺ ڽ  ε  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void rasterTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.chart3DControl.RasterType = (RasterType)this.rasterTypeComboBox.SelectedIndex;
        }

        #endregion
        #region ġ ʱȭ ư Ŭ óϱ - resetPositionButton_Click(sender, e)

        /// <summary>
        /// ġ ʱȭ ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void resetPositionButton_Click(object sender, EventArgs e)
        {
            this.chart3DControl.SetCoefficients(1350, 70, 230);
        }

        #endregion
        #region ̹  ư Ŭ óϱ - saveImageButton_Click(sender, e)

        /// <summary>
        /// ̹  ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void saveImageButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();

            saveFileDialog.Title      = "PNG ̹ ϱ";
            saveFileDialog.Filter     = "PNG ̹|*.png";
            saveFileDialog.DefaultExt = ".png";

            if(DialogResult.Cancel == saveFileDialog.ShowDialog(this))
            {
                return;
            }

            Bitmap bitmap = this.chart3DControl.GetBitmap();

            try
            {
                bitmap.Save(saveFileDialog.FileName, ImageFormat.Png);
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Լ ϱ 1 - SetFunction1()

        /// <summary>
        /// Լ ϱ 1
        /// </summary>
        private void SetFunction1()
        {
            RendererDelegate rendererDelegate = delegate(double X, double Y)
            {
                double r = 0.15 * Math.Sqrt(X * X + Y * Y);

                if(r < 1e-10)
                {
                    return 120;
                }
                else
                {
                    return 120 * Math.Sin(r) / r;
                }
            };

            this.chart3DControl.SetFunction
            (
                rendererDelegate,
                new PointF(-120f, -80f),
                new PointF(120f, 80f),
                5d,
                NormalizeType.MaintainXYZ
            );
        }

        #endregion
        #region Լ ϱ 2 - SetFunction2()

        /// <summary>
        /// Լ ϱ 2
        /// </summary>
        private void SetFunction2()
        {
            string formula = "12 * sin(x) * cos(y) / (sqrt(sqrt(x * x + y * y)) + 0.2)";

            try
            {
                RendererDelegate rendererDelegate = FunctionCompiler.Compile(formula);

                this.chart3DControl.SetFunction
                (
                    rendererDelegate,
                    new PointF(-10f, -10f),
                    new PointF(10f, 10f),
                    0.5d,
                    NormalizeType.MaintainXYZ
                );
            }
            catch(Exception exception)
            {
                MessageBox.Show(exception.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region ǥ Ʈ ϱ - SetSurfacePoints()

        /// <summary>
        /// ǥ Ʈ ϱ
        /// </summary>
        private void SetSurfacePoints()
        {
            int[,] valueArray = new int[,]
            {
                { 34767, 34210, 33718, 33096, 32342, 31851, 31228, 30867, 31457, 30867, 30266, 28934, 27984, 26492, 25167, 25167, 25167},
                { 34669, 34210, 33653, 33096, 32539, 32047, 31490, 30933, 31293, 30671, 29983, 28803, 27886, 26492, 25167, 25167, 25167},
                { 34603, 34144, 33718, 33227, 32768, 32342, 31719, 30999, 31228, 30333, 29622, 28606, 27886, 26492, 25167, 25167, 25167},
                { 34472, 34079, 33653, 33161, 32768, 32408, 31785, 31162, 30802, 30048, 29360, 28312, 27755, 26367, 25049, 25049, 25049},
                { 34210, 33784, 33423, 33161, 32801, 32408, 31785, 31097, 30474, 29622, 29000, 28115, 27623, 26367, 25049, 25049, 25049},
                { 33980, 33587, 33161, 32935, 32588, 32342, 31621, 30802, 29852, 29000, 28377, 27689, 27421, 26367, 25049, 25049, 25049},
                { 33522, 33227, 32702, 32615, 32452, 31851, 30933, 30179, 29295, 28358, 27984, 27132, 27301, 26367, 25049, 25049, 25049},
                { 32672, 32178, 31916, 31469, 31246, 30540, 29852, 29065, 28377, 27623, 27263, 26706, 26935, 26367, 25049, 25049, 25049},
                { 30769, 30423, 29917, 29231, 29392, 28705, 28075, 27726, 27263, 26691, 26417, 26182, 26575, 26575, 25246, 25246, 25246},
                { 27525, 27518, 26701, 27334, 27682, 27402, 26903, 26707, 26444, 25887, 25719, 25690, 26122, 26122, 26122, 26122, 26122},
                { 23475, 23888, 24478, 25330, 26212, 26199, 25701, 25664, 25740, 25013, 24904, 25068, 25374, 25374, 25374, 25374, 25374},
                { 20677, 21445, 22544, 23593, 24441, 24785, 24538, 24644, 24773, 24299, 24062, 24576, 24510, 24510, 24510, 24510, 24510},
                { 18743, 19792, 20808, 22086, 22805, 23167, 23486, 23366, 23757, 23411, 23691, 23822, 23822, 23822, 23822, 23822, 23822},
                { 17334, 18579, 19497, 20775, 21463, 21848, 22288, 22446, 22643, 22446, 22643, 22708, 23069, 23069, 23069, 23069, 23069},
                { 16155, 17236, 18350, 19399, 20251, 20677, 21016, 21332, 21660, 21791, 21889, 21955, 22217, 22217, 22217, 22217, 22217},
                { 14746, 15860, 17039, 17990, 18842, 19595, 20050, 20349, 20546, 20840, 20972, 20972, 21332, 21332, 21332, 21332, 21332},
                { 13337, 14516, 15729, 16679, 17564, 18514, 18907, 19169, 19399, 19661, 19792, 19594, 20152, 20152, 20152, 20152, 20152},
                { 12452, 13435, 14615, 15499, 16253, 17105, 17596, 17924, 18153, 18285, 18428, 18776, 19104, 19104, 19104, 19104, 19104},
                { 11469, 12354, 13533, 14287, 15008, 15925, 16187, 16482, 16690, 16976, 17105, 17302, 17531, 17531, 17531, 17531, 17531},
                { 10486, 11370, 12255, 13009, 13861, 14746, 15172, 15368, 15434, 15630, 15794, 15991, 16351, 16351, 16351, 16351, 16351},
                {  9684, 10387, 11141, 11796, 12546, 13337, 14029, 14320, 14549, 14811, 14939, 15434, 15794, 15794, 15794, 15794, 15794},
                {  9059,  9634, 10617, 11141, 11838, 12681, 13411, 13861, 14121, 14624, 14868, 15172, 15368, 15368, 15368, 15368, 15368}
            };

            Point3D[,] pointArray = new Point3D[valueArray.GetLength(0), valueArray.GetLength(1)];

            for(int x = 0; x < valueArray.GetLength(0); x++)
            {
                for(int y = 0; y < valueArray.GetLength(1); y++)
                {
                    pointArray[x,y] = new Point3D(x * 10, y * 500, valueArray[x, y]);
                }
            }

            this.chart3DControl.AxisXLegend = "MAP (kPa)";
            this.chart3DControl.AxisYLegend = "Engine Speed (rpm)";
            this.chart3DControl.AxisZLegend = "ADS";

            this.chart3DControl.SetSurfacePoints(pointArray, NormalizeType.Separate);
        }

        #endregion
        #region ĳ  ϱ - SetScatterPoints(drawLine)

        /// <summary>
        /// ĳ  ϱ
        /// </summary>
        /// <param name="drawLine"> ׸ </param>
        private void SetScatterPoints(bool drawLine)
        {
            List<Scatter> scallterList = new List<Scatter>();

            for(double p = -22d; p < 22d; p += 0.1d)
            {
                double x = Math.Sin(p) * p;
                double y = Math.Cos(p) * p;
                double z = p;

                if(z > 0d)
                {
                    z /= 3d;
                }

                scallterList.Add(new Scatter(x, y, z, null));
            }

            if(drawLine)
            {
                this.chart3DControl.SetScatterLines(scallterList.ToArray(), NormalizeType.Separate, 3f);
            }
            else
            {
                this.chart3DControl.SetScatterPoints(scallterList.ToArray(), NormalizeType.Separate);
            }
        }

        #endregion
        #region ĳ Ʈ ϱ - SetScatterPoints()

        /// <summary>
        /// ĳ Ʈ ϱ
        /// </summary>
        private void SetScatterPoints()
        {
            List<Scatter> scatterList = new List<Scatter>();

            double x = 0.0d;
            double z = 0.0d;

            for(double p = 0.0d; p <= Math.PI * 1.32d; p += 0.025d)
            {
                x = Math.Cos(p) * 1.5d - 1.5d;
                z = Math.Sin(p) * 3.0d + 6.0d;

                scatterList.Add(new Scatter( x, -x, z, Brushes.Red));
                scatterList.Add(new Scatter(-x,  x, z, Brushes.Red));
            }

            double deltaX = x / 70d;
            double deltaZ = z / 70d;

            while(z >= 0.0d)
            {
                scatterList.Add(new Scatter( x, -x, z, Brushes.Red));
                scatterList.Add(new Scatter(-x,  x, z, Brushes.Red));

                x -= deltaX;
                z -= deltaZ;
            }

            this.chart3DControl.SetScatterPoints(scatterList.ToArray(), NormalizeType.MaintainXYZ);
        }

        #endregion
    }
}